<?php

/**********
 * STEP 3
 * Project Environments
 *********/

/**
 * STEP 3: Form
 */
function devshop_project_create_step_environments($form, &$form_state) {
  $project = &$form_state['project'];
  if ($project->verify_task_status == HOSTING_TASK_QUEUED || $project->verify_task_status == HOSTING_TASK_PROCESSING) {
    $note = '<p class="wait">' . t('Please wait while we connect to your repository and determine any branches.') . '</p>';
    $form['note'] = array(
      '#markup' => $note,
    );
    $form['#no_next'] = TRUE;

    // Add code to reload the page when complete.
    devshop_form_reloader($form, 'project');
    return $form;
  }

  $form['nid'] = array(
    '#type' => 'value',
    '#value' => $project->nid,
  );
  $form['project'] = array(
    '#tree' => TRUE,
  );
  $form['project']['environments'] = array(
    '#theme' => 'devshop_projects_create_settings_form',
    '#tree' => TRUE,
    '#prefix' => '<div id="environments-wrapper">',
    '#suffix' => '</div>',
  );

  // Ensure a blank row exists (happens when using 'Back' button).
  if (!isset($project->environments['NEW']) || !is_array($project->environments['NEW'])) {
    $project->environments['NEW'] = array();
  }
  foreach ($project->environments as $name => $environment) {
    // No platforms exist yet
    if ($name == 'NEW') {
      $env_title = '';
    }
    else {
      $env_title = $name;
    }

    $form['project']['environments'][$name] = array(
      '#tree' => TRUE,
      '#type' => 'fieldset',
      '#theme' => 'devshop_projects_settings_table',
    );

    // Environment properties
    // Generate field prefix and suffix using domain name pattern.
    if (variable_get('devshop_projects_allow_custom_base_url')) {
      $pattern = $project->base_url;
    }
    else {
      $pattern = variable_get('devshop_project_environment_url_pattern', '@project.@environment.@hostname');
    }
  
    $labels = explode('@environment', strtr($pattern, array(
      '@project' => $project->name,
      '@hostname' => $_SERVER['SERVER_NAME'],
    )));
    $form['project']['environments'][$name]['name'] = array(
      '#type' => 'textfield',
      '#default_value' => $env_title,
      '#size' => 6,
      '#maxlength' => 64,
      '#attributes' => array(
        'placeholder' => t('name'),
      ),
      '#field_prefix' => '<div class="input-group"><div class="input-group-addon">http://' . $labels[0] . '</div>',
      '#field_suffix' => '<div class="input-group-addon input-group-addon-url">' . $labels[1] .'</div>
        </div>',
    );
    $form['project']['environments'][$name]['site'] = array(
      '#type' => 'value',
      '#value' => isset($environment->site) ? $environment->site : NULL,
    );
    $form['project']['environments'][$name]['platform'] = array(
      '#type' => 'value',
      '#value' => isset($environment->platform) ? $environment->platform : NULL,
    );
    $form['project']['environments'][$name]['git_ref'] = array(
      '#type' => 'select',
      '#options' => devshop_projects_git_ref_options($project),
      '#default_value' => isset($environment->git_ref) ? $environment->git_ref : 'HEAD',
    );

    // HTTP Server select.
    $http_servers = hosting_get_servers('http');

    // Determine default web server.
    $default_web_server = isset($environment->settings, $environment->settings->web_server) ?
      $environment->settings->web_server :
      $project->settings->default_environment['web_server'];

    // Fallback if environment or default not found.
    if (empty($default_web_server)) {
      $default_web_server = key($http_servers);
    }

    if (count($http_servers) == 1) {
      $form['project']['environments'][$name]['settings']['web_server'] = array(
          '#type' => 'value',
          '#value' => $default_web_server,
      );
    }
    elseif (count($http_servers) > 1) {
      $form['project']['environments'][$name]['settings']['web_server'] = array(
        '#title' => t('Web server'),
        '#type' => 'select',
        '#options' => $http_servers,
        '#default_value' => $default_web_server
      );
    }

    // DB Server select.
    $db_servers = hosting_get_servers('db');

    // Determine default db server.
    $default_db_server = isset($environment->settings, $environment->settings->db_server) ?
      $environment->settings->db_server :
      $project->settings->default_environment['db_server'];

    // Fallback if environment or default not found.
    if (empty($default_db_server) || !isset($db_servers[$default_db_server])) {
      $default_db_server = key($db_servers);
    }

    if (count($db_servers) == 1) {
      $form['project']['environments'][$name]['settings']['db_server'] = array(
          '#type' => 'value',
          '#default_value' => $default_db_server,
      );
    }
    elseif (count($db_servers) > 1) {
      $form['project']['environments'][$name]['settings']['db_server'] = array(
        '#title' => t('Database server'),
        '#type' => 'select',
        '#options' => $db_servers,
        '#default_value' => $default_db_server
      );
    }

    // Add Environment
    $form['add_environment'] = array(
      '#type' => 'submit',
      '#value' => t('Add environment'),
      '#name' => 'add_environment',
      '#prefix' => '<div class="project-add-environment-button">',
      '#suffix' => '</div>',
    );
  }
  $form['project']['environments']['NEW']['name']['#attributes']['autofocus'] = 'autofocus';
  return $form;
}


/**
 * STEP 3: Validate
 */
function devshop_project_create_step_environments_validate(&$form, &$form_state) {
  $project = &$form_state['project'];

  $values = &$form_state['values'];

  if (isset($values['project'])) {

    // Check environment titles
    foreach ($values['project']['environments'] as $env => $env_settings) {
      // Check for illegal chars
      if ($env != 'NEW' && !empty($env_settings['name'])) {
        if (!preg_match('!^[a-z0-9_]+$!', $env_settings['name'])) {
          $form_item = 'environments][' . $env . '][name';
          form_set_error($form_item, t('The environment name must contain only lowercase letters, numbers, and underscores.'));
        }
      }

      // Check for unique names.
      if ($env == 'NEW' && array_key_exists($env_settings['name'], $values['project']['environments'])) {
        $form_item = 'environments][' . $env . '][name';
        form_set_error($form_item, t('The environment name must be unique.'));
      }
      if ($env != $env_settings['name'] && array_key_exists($env_settings['name'], $values['project']['environments'])) {
        $form_item = 'environments][' . $env . '][name';
        form_set_error($form_item, t('The environment name must be unique.'));
      }

    }

    // Reject if empty
    if (count($values['project']['environments']) < 1) {
      if ($form_state['clicked_button']['#name'] == 'add_environment') {
        form_set_error('project][environments][NEW][name', t('Name this environment before you add another.'));
      }
      else {
        form_set_error('project][environments][NEW][name', t('You must add at least one environment.'));
      }
    }
  }


}

///**
// * Functionality for add a new environment.
// */
//function devshop_projects_save_environments($form, &$form_state) {
//
//  $environments = $form_state['values']['project']['environments'];
//
//  if (!empty($environments['NEW']['name'])) {
//    $new_environment_name = $environments['NEW']['name'];
//    $environments[$new_environment_name] = $environments['NEW'];
//  }
//  unset($environments['NEW']);
//
//  // Delete all environments
//  db_delete('hosting_devshop_project_environment')
//    ->condition('project_nid', $form_state['values']['nid'])
//    ->execute();
//
//  // Create all environments
//  foreach ($environments as $environment) {
//    $name = trim($environment['name']);
//    if (!empty($name)) {
//      $id = db_insert('hosting_devshop_project_environment')
//              ->fields(array(
//                'project_nid' => $form_state['values']['nid'],
//                'name' => $environment['name'],
//                'git_ref' => $environment['git_ref'],
//                'platform' => $environment['platform'],
//                'settings' => serialize($environment['settings']),
//              ))
//              ->execute();
//    }
//  }
//
//  // Go back to the same page.
//  if ($form_state['clicked_button']['#name'] == 'add_environment') {
//    drupal_goto('projects/add/environments');
//  }
//}

/**
 * STEP 3: SUBMIT
 */
function devshop_project_create_step_environments_submit(&$form, &$form_state) {
  // Get project and reset properties..
  $project_node = node_load($form_state['values']['nid'], NULL, TRUE);
  $project = $project_node->project;
  if (isset($form_state['project']->code_path) && empty($project->code_path)) {
    $project->code_path = $form_state['project']->code_path;
  }
  if (isset($form_state['project']->drupal_path) && empty($project->drupal_path)) {
    $project->drupal_path = $form_state['project']->drupal_path;
  }

  // Save environments data
  $environments = $form_state['values']['project']['environments'];
  if (!empty($environments['NEW']['name'])) {
    $new_environment_name = $environments['NEW']['name'];
    $environments[$new_environment_name] = $environments['NEW'];
  }
  unset($environments['NEW']);

  foreach ($environments as $name => $environment) {
    if ($name != $environment['name']) {
      $environments[$environment['name']] = $environment;
      unset($environments[$name]);
    }
  }

  // Delete all existing environments
  db_delete('hosting_devshop_project_environment')
    ->condition('project_nid', $project_node->nid)
    ->execute();

  // Create all environments
  foreach ($environments as $name => $environment) {
    $environment = (object) $environment;
    $environment->settings = (object) $environment->settings;
    $name = trim($environment->name);

    if (!empty($name)) {
      $id = db_insert('hosting_devshop_project_environment')
        ->fields(array(
          'project_nid' => $form_state['values']['nid'],
          'name' => $environment->name,
          'platform' => $environment->platform,
          'settings' => serialize($environment->settings),
        ))
        ->execute();
    }

    // Create platform node if we don't have one.
    if (empty($environment->platform)) {
      //If platform hasn't been created yet, build platform node
      $platform = new stdClass;
      $platform->type = 'platform';
      $platform->title = $project->name . '_' . $name;

      // Platform publish_path
  
      // If drupal path is set, repo path and platform path are different. Append drupal_path to repo path to create publish path.
      if ($project->drupal_path) {
        $platform->git['repo_path'] = $project->code_path . '/' . $environment->name;
        $platform->publish_path = $project->code_path . '/' . $environment->name . '/' . $project->drupal_path;
      }
      else {
        $platform->git['repo_path'] = $project->code_path . '/' . $environment->name;
        $platform->publish_path = $project->code_path . '/' . $environment->name;
      }

      // Git information
      $platform->git['repo_url'] = $project->git_url;
      $platform->git['git_ref'] = $environment->git_ref;

      // Other attributes
      $platform->web_server = $environment->settings->web_server;
      $platform->db_server = $environment->settings->db_server;

      // Save the node
      global $user;
      node_object_prepare($platform);
      $platform->language = LANGUAGE_NONE; // Or e.g. 'en' if locale is enabled
      $platform->uid = $user->uid;
      $platform->status = 1;
      $platform->promote = 0;
      $platform->comment = 0;
      $platform = node_submit($platform);
      node_save($platform);

      if (!$platform->nid) {
        form_set_error('projects', t('An unknown problem has occured: the platform node was not created for the %env environment.', array('%env' => $environment->name)));
      }
      $environment->platform = $platform->nid;

      // Update environment with our platform
      db_update('hosting_devshop_project_environment')
        ->fields(array(
          'platform' => $environment->platform,
        ))
        ->condition('project_nid', $form_state['values']['nid'])
        ->condition('name', $environment->name)
        ->execute();
    }
  }

  // Go back to the same page.
  if ($form_state['clicked_button']['#name'] == 'add_environment') {
    drupal_goto('projects/add/environments');
  }

  // For all removed platforms, trigger a delete task
  $removed_environments = array_diff_key($project->environments, $environments);
  foreach ($removed_environments as $environment_name => $settings) {
    $platform_nid = $settings->platform;
    $platform_node = node_load($platform_nid);

    // Determine what to do here based on task status...
    // if verify task hasn't even run yet (and has never run) we can just delete
    // the platform node.
    if (empty($platform_node->verified)) {
      node_delete($platform_node);
    }
    // Create 'delete' task for the removed platform
    else if ($platform_nid) {
      hosting_add_task($platform_nid, 'delete');
    }
  }


  // Remove default "task" messages.
  drupal_set_message('Environments saved! Now preparing codebases...');
}
